@@ -254,17 +254,16 @@ static int mystrncpyw(char *dest, const char *src, int n, int *maxwidth)
     return i;
 }
 
-int listBox(const char * text, int height, int width, poptContext optCon,
+int listBox(const char * text, int height, int width, int listHeight, poptContext optCon,
 		int flags, const char *default_item, char ** result) {
-    newtComponent form, okay, tb, answer, listBox;
+    newtComponent form = NULL, okay, tb, answer, listBox;
     newtComponent cancel = NULL;
     const char * arg;
     char * end;
-    int listHeight;
     int numItems = 0;
     int allocedItems = 5;
     int i, top;
-    int rc = DLG_OKAY;
+    int rc = DLG_ERROR;
     char buf[MAXBUF];
     int maxTagWidth = 0;
     int maxTextWidth = 0;
@@ -275,24 +274,26 @@ int listBox(const char * text, int height, int width, poptContext optCon,
 	const char * text;
 	const char * tag;
     } * itemInfo = malloc(allocedItems * sizeof(*itemInfo));
+    void * tmp;
 
-    if (itemInfo == NULL) return DLG_ERROR;
-    if (!(arg = poptGetArg(optCon))) return DLG_ERROR;
-    listHeight = strtoul(arg, &end, 10);
-    if (*end) return DLG_ERROR;
+    if (itemInfo == NULL)
+	goto error;
 
     while ((arg = poptGetArg(optCon))) {
 	if (allocedItems == numItems) {
 	    allocedItems += 5;
-	    itemInfo = realloc(itemInfo, sizeof(*itemInfo) * allocedItems);
-	    if (itemInfo == NULL) return DLG_ERROR;
+	    tmp = realloc(itemInfo, sizeof(*itemInfo) * allocedItems);
+	    if (tmp == NULL)
+		goto error;
+	    itemInfo = tmp;
 	}
 
 	itemInfo[numItems].tag = arg;
 	if (default_item && (strcmp(default_item, arg) == 0)) {
 	 	defItem = numItems;
 	}
-	if (!(arg = poptGetArg(optCon))) return DLG_ERROR;
+	if (!(arg = poptGetArg(optCon)))
+	    goto error;
 
 	if (!(flags & FLAG_NOITEM)) {
 	    itemInfo[numItems].text = arg;
@@ -307,7 +308,7 @@ int listBox(const char * text, int height, int width, poptContext optCon,
 	numItems++;
     }
     if (numItems == 0)
-	return DLG_ERROR;
+	goto error;
 
     if (flags & FLAG_NOTAGS) {
 	    maxTagWidth = 0;
@@ -326,7 +327,7 @@ int listBox(const char * text, int height, int width, poptContext optCon,
 	i = 2;
     }
 
-    lineWidth = min(maxTagWidth + maxTextWidth + i, SLtt_Screen_Cols - 10);
+    lineWidth = min(maxTagWidth + maxTextWidth + i + 1, SLtt_Screen_Cols - 6);
     listBox = newtListbox( (width - lineWidth) / 2 , top + 1, listHeight,
                           NEWT_FLAG_RETURNEXIT | scrollFlag);
 
@@ -335,12 +336,15 @@ int listBox(const char * text, int height, int width, poptContext optCon,
     if (maxTextWidth == 0) {
         tagWidth = lineWidth;
     } else {
-       if (maxTextWidth + maxTagWidth + i > lineWidth)
-               tagWidth = textWidth = (lineWidth / 2) - 2;
-       else {
-               tagWidth++;
-               textWidth++;
-       }
+	tagWidth++;
+	while (textWidth + tagWidth + i > lineWidth) {
+	    if (textWidth >= tagWidth && textWidth > 0)
+		textWidth--;
+	    else if (tagWidth > 0)
+		tagWidth--;
+	    else
+		break;
+	}
     }
 
     if (!(flags & FLAG_NOTAGS)) {
@@ -376,31 +380,35 @@ int listBox(const char * text, int height, int width, poptContext optCon,
     *result = NULL;
     if (answer == cancel)
 	rc = DLG_CANCEL;
-    if (answer == NULL)
+    else if (answer == NULL)
 	rc = DLG_ESCAPE;
     else {
 	i = (long) newtListboxGetCurrent(listBox);
 	*result = strdup(itemInfo[i].tag);
+	if (*result == NULL)
+	    goto error;
+	rc = DLG_OKAY;
     }
 
-    newtFormDestroy(form);
+error:
+    if (form)
+	newtFormDestroy(form);
     free(itemInfo);
 
     return rc;
 }
 
-int checkList(const char * text, int height, int width, poptContext optCon,
-		int useRadio, int flags, char *** selections) {
-    newtComponent form, okay, tb, subform, answer;
+int checkList(const char * text, int height, int width, int listHeight,
+	      poptContext optCon, int useRadio, int flags, char *** selections) {
+    newtComponent form = NULL, okay, tb, subform, answer;
     newtComponent sb = NULL, cancel = NULL;
     const char * arg;
     char * end;
-    int listHeight;
     int numBoxes = 0;
     int allocedBoxes = 5;
     int i;
     int numSelected;
-    int rc = DLG_OKAY;
+    int rc = DLG_ERROR;
     char buf[MAXBUF], format[MAXFORMAT];
     int maxWidth = 0;
     int top;
@@ -410,26 +418,34 @@ int checkList(const char * text, int height, int width, poptContext optCon,
 	newtComponent comp;
     } * cbInfo = malloc(allocedBoxes * sizeof(*cbInfo));
     char * cbStates = malloc(allocedBoxes * sizeof(*cbStates));
+    void * tmp;
 
-    if ( (cbInfo == NULL) || (cbStates == NULL)) return DLG_ERROR;
-    if (!(arg = poptGetArg(optCon))) return DLG_ERROR;
-    listHeight = strtoul(arg, &end, 10);
-    if (*end) return DLG_ERROR;
+    if (cbInfo == NULL || cbStates == NULL)
+	goto error;
 
     while ((arg = poptGetArg(optCon))) {
 	if (allocedBoxes == numBoxes) {
 	    allocedBoxes += 5;
-	    cbInfo = realloc(cbInfo, sizeof(*cbInfo) * allocedBoxes);
-	    cbStates = realloc(cbStates, sizeof(*cbStates) * allocedBoxes);
-	    if ((cbInfo == NULL) || (cbStates == NULL)) return DLG_ERROR;
+
+	    tmp = realloc(cbInfo, sizeof(*cbInfo) * allocedBoxes);
+	    if (tmp == NULL)
+		goto error;
+	    cbInfo = tmp;
+
+	    tmp = realloc(cbStates, sizeof(*cbStates) * allocedBoxes);
+	    if (tmp == NULL)
+		goto error;
+	    cbStates = tmp;
 	}
 
 	cbInfo[numBoxes].tag = arg;
-	if (!(arg = poptGetArg(optCon))) return DLG_ERROR;
+	if (!(arg = poptGetArg(optCon)))
+	    goto error;
 
 	if (!(flags & FLAG_NOITEM)) {
 	    cbInfo[numBoxes].text = arg;
-	    if (!(arg = poptGetArg(optCon))) return DLG_ERROR;
+	    if (!(arg = poptGetArg(optCon)))
+		goto error;
 	} else
 	    cbInfo[numBoxes].text = "";
 
@@ -460,16 +476,20 @@ int checkList(const char * text, int height, int width, poptContext optCon,
     subform = newtForm(sb, NULL, 0);
     newtFormSetBackground(subform, NEWT_COLORSET_CHECKBOX);
 
-    snprintf(format, MAXFORMAT, "%%-%ds  %%s", maxWidth);
+    if (flags & FLAG_NOTAGS)
+	snprintf(format, MAXFORMAT, "%%.0s%%s");
+    else
+	snprintf(format, MAXFORMAT, "%%-%ds  %%s", maxWidth);
+
     for (i = 0; i < numBoxes; i++) {
 	snprintf(buf, MAXBUF, format, cbInfo[i].tag, cbInfo[i].text);
 
 	if (useRadio)
-	    cbInfo[i].comp = newtRadiobutton(4, top + 1 + i, buf,
+	    cbInfo[i].comp = newtRadiobutton(2, top + 1 + i, buf,
 					cbStates[i] != ' ', 
 					i ? cbInfo[i - 1].comp : NULL);
 	else
-	    cbInfo[i].comp = newtCheckbox(4, top + 1 + i, buf,
+	    cbInfo[i].comp = newtCheckbox(2, top + 1 + i, buf,
 			      cbStates[i], NULL, cbStates + i);
 
 	newtCheckboxSetFlags(cbInfo[i].comp, NEWT_FLAG_RETURNEXIT, NEWT_FLAGS_SET);
@@ -487,14 +507,14 @@ int checkList(const char * text, int height, int width, poptContext optCon,
     *selections = NULL;
     if (answer == cancel)
 	rc = DLG_CANCEL;
-    if (answer == NULL)
+    else if (answer == NULL)
 	rc = DLG_ESCAPE;
     else {
 	if (useRadio) {
 	    answer = newtRadioGetCurrent(cbInfo[0].comp);
 	    *selections = malloc(sizeof(char *) * 2);
 	    if (*selections == NULL)
-		return DLG_ERROR;
+		goto error;
 	    (*selections)[0] = (*selections)[1] = NULL;
 	    for (i = 0; i < numBoxes; i++)
 		if (cbInfo[i].comp == answer) {
@@ -509,7 +529,7 @@ int checkList(const char * text, int height, int width, poptContext optCon,
 
 	    *selections = malloc(sizeof(char *) * (numSelected + 1));
 	    if (*selections == NULL)
-		return DLG_ERROR;
+		goto error;
 
 	    numSelected = 0;
 	    for (i = 0; i < numBoxes; i++) {
@@ -519,11 +539,15 @@ int checkList(const char * text, int height, int width, poptContext optCon,
 
 	    (*selections)[numSelected] = NULL;
 	}
+
+	rc = DLG_OKAY;
     }
 
+error:
     free(cbInfo);
     free(cbStates);
-    newtFormDestroy(form);
+    if (form)
+	newtFormDestroy(form);
 
     return rc;
 }
@@ -569,7 +593,7 @@ int messageBox(const char * text, int height, int width, int type, int flags) {
 	answer = newtFormGetCurrent(form);
 
 	if (answer == no)
-	    return DLG_CANCEL;
+	    rc = DLG_CANCEL;
     }
     else {
 	newtDrawForm(form);
